/************************************************************************
 *
 * COMMON.H - NRPE Common Include File
 * Copyright (c) 1999-2003 Ethan Galstad (nagios@nagios.org)
 *
 * Last Modified: 2004-05-14 by Andreas Ericsson <ae@op5.se>
 *
 * Changes:
 * Added structures for packetversion 1 and 3.
 * Version 3 doesn't exist yet, so we needn't be in a hurry to support it,
 * but I thought it best to make sure we
 * 
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ************************************************************************/

#include "config.h"

#define PROGRAM_VERSION "2.1"
#define MODIFICATION_DATE "2004-05-11"


#define TRUE 1
#define FALSE 0

#define OK 0
#define ERROR -1

/* the various modes. Insert new before STATE_OOB, which is boundary */
enum {
	STATE_OK = 0,
	STATE_WARNING,
	STATE_CRITICAL,
	STATE_UNKNOWN,
	STATE_OOB
};

#define DEFAULT_SOCKET_TIMEOUT	10	/* marked in seconds */

#define MAX_INPUT_BUFFER	2048	/* max size of most buffers we use */
#define MAX_FILENAME_LENGTH     256

#define MAX_HOST_ADDRESS_LENGTH	256	/* max size of a host address */

#define NRPE_HELLO_COMMAND      "_NRPE_CHECK"

#define MAX_COMMAND_ARGUMENTS   16


/**************** PACKET STRUCTURE DEFINITION **********/
#define QUERY_PACKET		1		/* id code for a packet containing a query */
#define	RESPONSE_PACKET		2		/* id code for a packet containing a response */

#define NRPE_PACKET_VERSION_1 1
#define NRPE_PACKET_VERSION_2 2
#define NRPE_PACKET_VERSION_3 3

/* the highest version number we support */
#define NRPE_MAX_VERSION 2

/* max query string length (i.e. command_name and arguments)
 * there is no such thing as MAX_RESPONSE_LEN (see packet_struct_v3) */
#define MAX_QUERY_LEN	1024

/* max amount of data we'll send in one query/response */
#define MAX_PACKETBUFFER_LENGTH	1024

/*
 * This is bad.
 * In v1, bitwidth is OS dependant.
 * In v2, packet_version and packet_type has shifted place from v1.
 * From v3 and forward, everything should be ok.
 */
struct packet_struct_v1 {
	int packet_type;
	int packet_version;
	int result_code;
	int buffer_length;
	char buffer[MAX_PACKETBUFFER_LENGTH];
};

struct packet_struct_v2 {
	int16_t packet_version;
	int16_t packet_type;
	u_int32_t crc32_value;
	int16_t result_code;
	char buffer[MAX_PACKETBUFFER_LENGTH];
};

#define PACKET_SIZE (sizeof(struct packet_struct_v2)-MAX_PACKETBUFFER_LENGTH)

/*
 * (in the future ...)
 * skip buffer and allow arbitrarily long response.
 * Put packet_version in unsigned char (always 8 bits) first,
 * so we know it will be the same on every OS and arch.
 * Use u_int32_t for result_code, so we can use it for VMS as well,
 * and without extra exception handling for return codes of -1 for
 * STATE_UNKNOWN
 * This version is not yet supported by check_nrpe, but adding support for
 * it is trivial.
 * Notice that we use buf_chunks, which makes it possible to support a
 * ariety of different cipher modes (many modes require static length data).
 */

#define BUF_CHUNK_SIZE 1024

struct packet_struct_v3 {
	unsigned char packet_version;	/* NRPE version */
	unsigned char packet_type;		/* response, query, control */
	unsigned char cipher;			/* cipher type: 0=plain, 1=ssl, ... */
	u_int32_t crc32_value;			/* checksum */
	u_int32_t result_code;			/* plugin return code. */
	u_int32_t buf_chunks;			/* 1K chunks of output */
};

typedef struct packet_struct_v2 packet;
typedef struct packet_struct_v1 packet_v1;
typedef struct packet_struct_v2 packet_v2;
typedef struct packet_struct_v3 packet_v3;
